/* TitraxConduit.h
   Header file for a DLL that can be used on windows systems to synchronize the
   the titrax palm program with the desktop system via the Palm CDK API's.

   Copyright Jim Weller 2004

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA. 
*/
#ifndef  __TITRAX_MAIN_CONDUIT__
#define  __TITRAX_MAIN_CONDUIT__

#include <windows.h>         // windows
#include <time.h>            // time
#include <strsafe.h>         // new buffer secure microsoft string functions

#include <Hsapi.h>           // Palm CDK: HotSync API
#include <syncmgr.h>         // Palm CDK: HotSync Manager
#include <PALM_CMN.H>        // Palm CDK: Palm Common
#include <HSLog.h>           // Palm CDK: HotSync Log
#include <condapi.h>         // Palm CDK: Conduit API

#include "resource.h"        // windows resources

#define TITRAX_CONDUIT_VERSION 0x00000104

/* windows garbage to make a function available in the DLL */
#ifndef ExportFunc
#define ExportFunc __declspec( dllexport )
#endif
#ifndef ImportFunc
#define ImportFunc __declspec( dllimport )
#endif


#define STRICT 1
#define ASSERT(f)          ((void)0)
#define TRACE0(sz)
#define TRACE(sz)

/* standard handles for DLLs*/
HANDLE hLangInstance;
HANDLE hAppInstance;
extern HANDLE hLangInstance;
extern HANDLE hAppInstance;

/* choices for user settings */
typedef enum titraxFileEnum { csv_file, txt_file } titraxFileType;
typedef enum titraxTimeEnum { time_excel, time_string, time_1900, time_1904, time_1970 } titraxTimeType;

/* actual user settings (read/written to/from registry) */
boolean titraxDumpSummary;
boolean titraxDumpHistory;
titraxFileType titraxFileTypeOption;
titraxTimeType titraxTimeTypeOption;


/* counter for dll access semantics */
static int iDllAttachCount = 0;

/* displays HotSync settings for titrax */
long CALLBACK HSConfigurationDialogProc(HWND hWnd, UINT Message, WPARAM wParam, LPARAM lParam);

/* a place to cache palm DB records as we read them */
#define kMaxRecordSize  65535
TCHAR   gBigBuffer[kMaxRecordSize];


/* functions that we export from the DLL */
extern "C" {
 
    /* for running the conduit.
       These are needed to be defined to implement the conduit interface*/
    ExportFunc long OpenConduit(PROGRESSFN, CSyncProperties&);
    ExportFunc long GetConduitName(char*,WORD);
    ExportFunc DWORD GetConduitVersion();
    ExportFunc long ConfigureConduit(CSyncPreference& pref);
    ExportFunc long GetConduitInfo(ConduitInfoEnum infoType, void *pInArgs, void *pOut, DWORD *dwOutSize);

}

/* Typedef to pass a pointer to a progress function */
typedef  long (*PROGRESSFN) (char*);


/* internal utility functions */

// load summaries
DWORD loadTitraxNames();
int loadTitraxRecords();

// output to file
DWORD titraxDumpHistoryToFile(TCHAR *);
DWORD titraxDumpSummaryToFile(TCHAR *);

// time conversions
time_t PalmTime2WinTime( DWORD  );
time_t WinTime2PalmTime( DWORD  );
double PalmTime2Excel1900TimeSerial( time_t );
double WinTime2Excel1900TimeSerial( time_t );
DWORD PalmTimeTo1900Time( DWORD );
void elapsedSecondsToDDDHHMMSS( DWORD , char*  );
double ElapsedTime2Excel1900TimeSerial( time_t  );

// windows helpers
boolean DoHelpFile(TCHAR*);
TCHAR* resString(UINT);

// read/load settings from the registry
DWORD titraxWriteRegistry();
DWORD titraxReadRegistry();



/*
******************************************************************************
*
* Titrax imported
*
* Much of this is converted from the titrax headers.
* I show the Titrax equivalents here. If I can get this package
* integrated w/ the Titrax source then it can just use the Titrax headers
* themselves. I've heavily commented to help me understand titrax
*
******************************************************************************
*/

/* this is the length of the database field for the project name. Can be set to
   arbitrary values, but higher values increase the size of the database. */
#define MAXTEXT 103

/* maximal number of project slots. Can be set to arbitrary values, but higher values
 increase the size of the database and the size of the used heap.*/
#define MAXSLOT 999

// database names. Technically this should be in a string resource, but
// for clarity...
#define titraxNameDBName "TitraxNameDB"
#define titraxSlotDBName "TitraxDataDB"
#define titraxNoteDBName "TitraxNoteDB"
#define titraxHistDBName "TitraxHistDB"

/* possible choices for the users time logging preference */
typedef enum loggingEnum {dontLog, toNote, toDatebook} loggingType;

/* this is the definition of a record in the TitraxDataDB. The
   original (from palm titrax) is shown below for reference. */
struct packedTitraxSlotDataType 
{
  DWORD elapsed;
  DWORD startedAt;
  BYTE active;
  BYTE logging;
  BYTE  ignored;
};

/*
typedef struct {
  UInt32 elapsed;
  UInt32 startedAt;
  Boolean active;
  loggingType logging;
  Boolean ignored;
} slotDataType;
*/


/* history data type. */
struct packedTitraxHistDataType {
    char  name[MAXTEXT+1];
    DWORD start;
    DWORD stop;
};

/* palm titrax history record
typedef struct {
  Char name[MAXTEXT+1];
  UInt32 start;
  UInt32 stop;
} histType;
*/


/*
******************************************************************************
*
* Titrax derived
*
* These data structures are designed to hold the data that comes
* out of the combined name,data, and note databases.
*
******************************************************************************
*/

/* This is a record designed for use on the desktop
   it is a combination of the Titrax's name, data, and note databases */
struct titraxRecordDataType {
    DWORD index;
    char  name[kMaxRecordSize];
    DWORD elapsed;
    DWORD startedAt;
    BYTE  active;
    BYTE  logging;
    BYTE  ignored;
    char  note[kMaxRecordSize];
};

/* This is a container for titraxName and titraxData records read from the 
handheld during hotsync. */
static titraxRecordDataType records[MAXSLOT];
static WORD recordCount;

#endif // __TITRAX_MAIN_CONDUIT__


